/***
 * Required libraries:
 * Arduino_GFX: https://github.com/moononournation/Arduino_GFX.git
 * libhelix: https://github.com/pschatzmann/arduino-libhelix.git
 * JPEGDEC: https://github.com/bitbank2/JPEGDEC.git
 */
#define MJPEG_FILENAME_1 "/video1.mjpeg"
#define MJPEG_FILENAME_2 "/video2.mjpeg"
#define MJPEG_FILENAME_3 "/video3.mjpeg"  
#define FPS 25
#define MJPEG_BUFFER_SIZE (240 * 240 * 2) // 16-bit color buffer

// SD卡引脚配置
#define SDMMC_D3 13
#define SDMMC_CMD 18
#define SDMMC_CLK 17
#define SDMMC_D0 21

// 显示屏引脚配置
#define TFT_MOSI 2
#define TFT_SCLK 4
#define TFT_CS 5
#define TFT_DC 47
#define TFT_RST 38
#define LCD_BL 42
#define BOOT_BTN 0  // BOOT按键

#include <Arduino_GFX_Library.h>
#include "MjpegClass.h"
#include <SD_MMC.h>

// 显示驱动配置
Arduino_DataBus *bus = new Arduino_ESP32SPI(TFT_DC, TFT_CS, TFT_SCLK, TFT_MOSI);
Arduino_GFX *gfx = new Arduino_GC9A01(bus, TFT_RST, 0, true, 240, 240);
MjpegClass mjpeg;

// 全局变量
static int next_frame = 0;
static int skipped_frames = 0;
static unsigned long start_ms, curr_ms, next_frame_ms;
static unsigned long total_read_video_ms = 0;
static unsigned long total_decode_video_ms = 0;
static unsigned long total_show_video_ms = 0;
volatile uint8_t currentVideo = 1;  // 修改为支持3个视频的索引 (1,2,3)
volatile bool buttonPressed = false;
unsigned long lastButtonPress = 0;
const unsigned long debounceDelay = 500;
uint8_t *mjpeg_buf = NULL; // 全局缓冲区指针
File vFile; // 全局文件对象

// 绘制回调函数
static int drawMCU(JPEGDRAW *pDraw) {
  unsigned long s = millis();
  gfx->draw16bitBeRGBBitmap(pDraw->x, pDraw->y, pDraw->pPixels, pDraw->iWidth, pDraw->iHeight);
  total_show_video_ms += millis() - s;
  return 1;
}

// 改进的按键检查函数
void IRAM_ATTR checkBootButton() {
  static unsigned long last_interrupt_time = 0;
  unsigned long interrupt_time = millis();
  
  if (interrupt_time - last_interrupt_time > debounceDelay) {
    buttonPressed = true;
    last_interrupt_time = interrupt_time;
  }
}

// 释放资源函数
void releaseResources() {
  if (mjpeg_buf) {
    free(mjpeg_buf);
    mjpeg_buf = NULL;
  }
  if (vFile) {
    vFile.close();
  }
}

// 获取当前视频文件名
const char* getCurrentVideoFilename() {
  switch(currentVideo) {
    case 1: return MJPEG_FILENAME_1;
    case 2: return MJPEG_FILENAME_2;
    case 3: return MJPEG_FILENAME_3;
    default: return MJPEG_FILENAME_1;
  }
}

// 播放视频函数
void playVideo() {
  const char *filename = getCurrentVideoFilename();
  
  // 释放之前分配的资源
  releaseResources();
  
  vFile = SD_MMC.open(filename);
  if (!vFile || vFile.isDirectory()) {
    Serial.printf("ERROR: Failed to open %s\n", filename);
    gfx->fillScreen(BLACK);
    gfx->setTextColor(0xFFFF);
    gfx->setCursor(10, 10);
    gfx->printf("Error: %s", filename);
    delay(2000);
    return;
  }

  // 分配缓冲区
  mjpeg_buf = (uint8_t *)malloc(MJPEG_BUFFER_SIZE);
  if (!mjpeg_buf) {
    Serial.println("Buffer alloc failed");
    vFile.close();
    return;
  }

  Serial.printf("Playing: %s\n", filename);
  if (!mjpeg.setup(&vFile, mjpeg_buf, drawMCU, true, 0, 0, gfx->width(), gfx->height())) {
    Serial.println("MJPEG setup failed");
    releaseResources();
    return;
  }

  // 初始化计时器
  start_ms = millis();
  curr_ms = millis();
  next_frame = 0;
  next_frame_ms = start_ms + (next_frame * 1000 / FPS);
  
  while (vFile.available() && mjpeg.readMjpegBuf()) {
    // 检查按键状态
    if (buttonPressed) {
      buttonPressed = false;
      currentVideo = (currentVideo % 3) + 1; // 循环切换1→2→3→1
      break;
    }

    // 帧率控制逻辑
    curr_ms = millis();
    total_read_video_ms += curr_ms - start_ms;

    if (curr_ms < next_frame_ms) {
      if (!mjpeg.drawJpg()) {
        Serial.println("Draw JPG failed");
        break;
      }
      total_decode_video_ms += millis() - curr_ms;
    } else {
      ++skipped_frames;
      Serial.println("Skip frame");
    }

    // 等待下一帧时间
    while (millis() < next_frame_ms) {
      delay(1);
      if (buttonPressed) break;
    }

    if (buttonPressed) {
      buttonPressed = false;
      currentVideo = (currentVideo % 3) + 1; // 循环切换1→2→3→1
      break;
    }

    // 更新下一帧时间
    next_frame++;
    next_frame_ms = start_ms + (next_frame * 1000 / FPS);
  }

  // 清屏避免残影
  gfx->fillScreen(BLACK); 
  Serial.println("Playback ended");
}

void setup() {
  Serial.begin(115200);
  while(!Serial); // 等待串口连接

  // 初始化BOOT按键
  pinMode(BOOT_BTN, INPUT_PULLUP);
  attachInterrupt(digitalPinToInterrupt(BOOT_BTN), checkBootButton, FALLING);

  // 初始化显示屏
  Serial.println("Initializing display...");
  if (!gfx->begin(80000000)) {
    Serial.println("Display init failed!");
    while(1);
  }
  gfx->fillScreen(BLACK);
  gfx->setTextColor(0xFFFF);
  gfx->setCursor(10, 10);
  gfx->println("Display Ready");
  delay(1000);

  // 初始化SD卡
  Serial.println("Mounting SD card...");
  pinMode(SDMMC_D3, OUTPUT);
  digitalWrite(SDMMC_D3, HIGH);
  SD_MMC.setPins(SDMMC_CLK, SDMMC_CMD, SDMMC_D0);
  
  if (!SD_MMC.begin("/sdcard", true, true, 8000000)) {
    Serial.println("SD Card Mount Failed");
    gfx->fillScreen(BLACK);
    gfx->setCursor(10, 10);
    gfx->println("SD Card Error!");
    while(1);
  }

  // 检查视频文件
  Serial.println("Checking video files...");
  if (!SD_MMC.exists(MJPEG_FILENAME_1)) {
    Serial.printf("%s not found!\n", MJPEG_FILENAME_1);
    gfx->fillScreen(BLACK);
    gfx->setCursor(10, 10);
    gfx->printf("%s missing", MJPEG_FILENAME_1);
    while(1);
  }
  
  if (!SD_MMC.exists(MJPEG_FILENAME_2)) {
    Serial.printf("%s not found!\n", MJPEG_FILENAME_2);
    gfx->fillScreen(BLACK);
    gfx->setCursor(10, 10);
    gfx->printf("%s missing", MJPEG_FILENAME_2);
    while(1);
  }

  if (!SD_MMC.exists(MJPEG_FILENAME_3)) {  // 新增第三个视频文件检查
    Serial.printf("%s not found!\n", MJPEG_FILENAME_3);
    gfx->fillScreen(BLACK);
    gfx->setCursor(10, 10);
    gfx->printf("%s missing", MJPEG_FILENAME_3);
    while(1);
  }

  // 背光控制
  pinMode(LCD_BL, OUTPUT);
  digitalWrite(LCD_BL, LOW); //
  
  Serial.println("System Ready");
}

void loop() {
  playVideo();  // 统一播放函数
  
  // 添加短暂延迟防止快速切换
  delay(200);
}